import "dart:convert";
import "dart:mirrors";
import "../helpers/annotations.dart";
import "contracts/iserializer.dart";

class JsonSerializer implements ISerializer {
  dynamic deserialize(Type entityType, String json) {
    final Map jsonMap = JSON.decode(json);
    final typeMirror = reflectClass(entityType);
    final obj = typeMirror.newInstance(new Symbol(""), const []);
    jsonMap.forEach((k, v) {
      final fieldName = new Symbol(k);
      final fieldTypeKey =
          typeMirror.declarations.keys.firstWhere((key) => key == fieldName);
      final fieldTypeMirror =
          (typeMirror.declarations[fieldTypeKey] as VariableMirror).type;
      obj.setField(fieldName, _deserialize(fieldTypeMirror, v));
    });
    return obj.reflectee;
  }

  String serialize(dynamic object) {
    final instanceMirror = reflect(object);
    final classMirror = instanceMirror.type;
    final map = {};
    for (VariableMirror field
        in classMirror.declarations.values.where((t) => t is VariableMirror)) {
      var fieldNameAsSerialized = (field.metadata
              .firstWhere((m) => m.reflectee.runtimeType == Serialize,
                  orElse: () => null)
              ?.reflectee as Serialize)
          ?.name;
      if (fieldNameAsSerialized != null) {
        final key = fieldNameAsSerialized == ""
            ? MirrorSystem.getName(field.simpleName)
            : fieldNameAsSerialized;
        map[key] =
            _serialize(field.type, instanceMirror.getField(field.simpleName));
      }
    }
    return new JsonEncoder.withIndent("    ").convert(map);
  }

  dynamic _deserialize(TypeMirror instanceTypeMirror, dynamic v) {
    if (instanceTypeMirror.isAssignableTo(reflectType(ISerializable))) {
      return deserialize(instanceTypeMirror.reflectedType, v);
    } else if (instanceTypeMirror.isAssignableTo(reflectType(List))) {
      return v
          .map((x) => _deserialize(instanceTypeMirror.typeArguments.first, x))
          .toList();
    } else if (reflectClass(instanceTypeMirror.reflectedType).isEnum) {
      return _enumFromInt(instanceTypeMirror.reflectedType, v);
    } else {
      return v;
    }
  }

  dynamic _serialize(TypeMirror instanceType, InstanceMirror instanceMirror) {
    if (instanceType.isAssignableTo(reflectType(ISerializable))) {
      return serialize(instanceMirror.reflectee); // poate tre sa trimit si tip
    } else if (instanceType.isAssignableTo(reflectType(List))) {
      return instanceMirror.reflectee
          .map((e) => _serialize(instanceType.typeArguments.first, reflect(e)))
          .toList();
    } else if (reflectClass(instanceType.reflectedType).isEnum) {
      return _enumToInt(instanceType.reflectedType, instanceMirror.reflectee);
    } else {
      return instanceMirror.reflectee;
    }
  }

  dynamic _enumFromInt(Type t, int value) {
    var values =
        (reflectType(t) as ClassMirror).getField(#values).reflectee as List;
    if (value == -1 || value >= values.length) {
      return null;
    }
    return values[value];
  }

  int _enumToInt(Type t, dynamic value) =>
      ((reflectType(t) as ClassMirror).getField(#values).reflectee as List)
          .indexOf(value);
}
