import "package:test/test.dart";
import "../lib/src/helpers/annotations.dart";
import "../lib/src/services/contracts/iserializer.dart";
import "../lib/src/services/json_serializer.dart";

enum CareerStage { Junior, Middle, Senior }

class Employee implements ISerializable {
  @serialize
  String name;
  @serialize
  int age;
  @serialize
  CareerStage stage;

  @override
  String toString() => "$name $age $stage";
}

class Office implements ISerializable {
  @serialize
  List<Employee> employees;

  @override
  String toString() => "employees : ${employees.join(", ")}";
}

class ContactsTable implements ISerializable {
  @serialize
  List<List<Employee>> contactsLists;
}

void main() {
  group("toJson tests", () {
    ISerializer serializer = new JsonSerializer();
    Employee employee;
    Office office;
    ContactsTable table;
    setUp(() {
      employee = new Employee()
        ..name = "John Smith"
        ..age = 42
        ..stage = CareerStage.Middle;
      office = new Office()..employees = new List<Employee>();
      office.employees.add(employee);
      office.employees.add(employee);
      office.employees.add(employee);
      table = new ContactsTable()..contactsLists = new List<List<Employee>>();
      table.contactsLists.add(office.employees);
      table.contactsLists.add(office.employees);
      table.contactsLists.add(office.employees);
    });

    test("Entity can serialize PODOs", () {
      var serializedEmployee = serializer.serialize(employee);
      var deserializedEmployee =
          serializer.deserialize(Employee, serializedEmployee);
      expect(employee.toString(), equals(deserializedEmployee.toString()));
    });

    test("Entity can serialize null enums", () {
      employee.stage = null;
      var serializedEmployee = serializer.serialize(employee);
      var deserializedEmployee =
          serializer.deserialize(Employee, serializedEmployee);
      expect(employee.toString(), equals(deserializedEmployee.toString()));
    });

    test("Entity can serialize PODO lists", () {
      var serializedOffice = serializer.serialize(office);
      var deserializedOffice = serializer.deserialize(Office, serializedOffice);
      expect(office.toString(), equals(deserializedOffice.toString()));
    });

    test("Entity can serialize list of lists of PODOs", () {
      var serializedTable = serializer.serialize(table);
      var deserializedTable =
          serializer.deserialize(ContactsTable, serializedTable);
      print(serializedTable);
      expect(table.toString(), equals(deserializedTable.toString()));
    });
  });
}
